/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CreamProxy_Delegate: the implementation of Delegation class for the CREAM PROXY
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CreamProxy_Delegate.cpp,v 1.10.4.2 2008/08/25 14:06:38 adorigo Exp $
*/

#include "CREAM_CLIENTH.h"
#include "CREAM_CLIENTStub.h"

#include "glite/ce/cream-client-api-c/CreamProxy_Impl.h"
#include "glite/ce/cream-client-api-c/certUtil.h"
#include "glite/ce/cream-client-api-c/VOMSWrapper.h"
#include "glite/ce/cream-client-api-c/creamApiLogger.h"

#include <ctime>

using namespace std;
using namespace glite::ce::cream_client_api::soap_proxy;

//______________________________________________________________________________
CreamProxy_Delegate::CreamProxy_Delegate( const string& delegID, /*string* S_certtxt,*/ const int timeout ) throw() 
 : AbsCreamProxy( timeout ), m_delegation_ID( delegID )//, m_certtxt( S_certtxt )
{
  //m_delegation_ID = "";
  //m_certtxt = NULL;
}

//______________________________________________________________________________
void CreamProxy_Delegate::execute(const string& serviceURL) 
  throw(BaseException&,
	InvalidArgumentException&,
	GridProxyDelegationException&,
	JobSubmissionDisabledException&,
	GenericException&,
	//	AuthenticationException&,
	AuthorizationException&,
	DelegationException&,
	InternalException&,
	ConnectionTimeoutException&,
	auth_ex&,
	soap_ex&,
	soap_runtime_ex&)
{
   char *certtxt;
   
   CREAM_SAFE_LOG(*m_log_dev << log4cpp::Priority::INFO
                  << "CreamProxy_Delegate::execute() - Calling "
		  << "getProxyReq to remote service ["
                  << serviceURL << "]" );

    {
      initSoap init(this);

      delegation2__getProxyReqResponse proxyReqResp;
      {
#ifdef DOPROFILE
        apiUtil::scoped_timer T("CreamProxy_Delegate::execute()-soap_call_delegation2__getProxyReq");
#endif
        if(SOAP_OK != soap_call_delegation2__getProxyReq(m_soap,
                                                        serviceURL.c_str(),
                                        		NULL,    // soapAction
                                        		m_delegation_ID,
                                        		proxyReqResp))
          ExceptionFactory::raiseException(m_soap); 
      }
    
      CREAM_SAFE_LOG(*m_log_dev << log4cpp::Priority::INFO
                     << "CreamProxy_Delegate::execute() - "
	             << "Signing returned proxy with our proxy certificate ["
                     << m_certfile<< "]..." );

      //time_t timeleft = certUtil::getProxyTimeLeft(m_certfile);

      VOMSWrapper V( m_certfile );
      if( !V.IsValid( ) ) {
        string errmex = "Authentication error: ";
        errmex += V.getErrorMessage();
        throw( auth_ex( errmex ));
      }

      long int timeleft = V.getProxyTimeEnd() - time(NULL);

      if(timeleft <= 0 ) {
        throw auth_ex("Proxy certificate is expired! Cannot make delegation");
      }

      if (certUtil::GRSTx509MakeProxyCert_local(&certtxt,
                                      stderr,
                                      (char*)proxyReqResp._getProxyReqReturn.c_str(),
                                      (char*)m_certfile.c_str(),
                                      (char*)m_certfile.c_str(),
                                      timeleft/60))
        throw DelegationException("Delegation proxy make failed: GRSTx509MakeProxyCert returned error") ;
    } // calls the my_authn's dtor that clears the SOAP runtime
    
    //S_certtxt = string(certtxt);
//    *m_certtxt = string(certtxt);
    
    CREAM_SAFE_LOG(*m_log_dev<< log4cpp::Priority::INFO  
    		   << "CreamProxy_Delegate::execute() - "
		   << "Sending signed proxy to ["
             	   << serviceURL << "] with id ["<<m_delegation_ID<<"]"
		   );
    {
      initSoap init(this);
      delegation2__putProxyResponse response;

#ifdef DOPROFILE
      apiUtil::scoped_timer T("CreamProxy_Delegate::execute()-soap_call_delegation2__putProxy");
#endif    
  
      if(SOAP_OK != soap_call_delegation2__putProxy(m_soap,
                                     		    serviceURL.c_str(),
                                     		    NULL,    // soapAction
                                     		    m_delegation_ID,
						    certtxt,
						    response))
        ExceptionFactory::raiseException(m_soap);
    }
}
