/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CreamProxy_Info: the implementation of JobInfo class for the CREAM PROXY
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CreamProxy_Info.cpp,v 1.16.2.3 2008/08/25 14:06:38 adorigo Exp $
*/

#include "CREAM_CLIENTH.h"
#include "CREAM_CLIENTStub.h"

#include "glite/ce/cream-client-api-c/CreamProxy_Impl.h"
#include "glite/ce/cream-client-api-c/creamApiLogger.h"
#include "glite/ce/cream-client-api-c/JobInfoWrapper.h"

using namespace glite::ce::cream_client_api::soap_proxy;
using namespace std;

//____________________________________________________________________________
void processorInfoResult::operator()( const CREAMTYPES__JobInfoResult* jr ) 
{
  if(!jr){
    throw InternalException("processorInfoResult::operator() - Pointer CREAMTYPES__JobInfoResult is NULL! Probable de-serialization error");
  }
  
  if(!jr->jobDescriptionId){
    throw InternalException("processorInfoResult::operator() - Pointer CREAMTYPES__JobInfoResult->jobDescriptionId is NULL! Probable de-serialization error");
  }
  
  string errMex;
  
  switch( jr->__union_2) {
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_2_JobUnknownFault:
    if( jr->union_2.JobUnknownFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_2.JobUnknownFault );
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobInfoWrapper::JOBUNKNOWN, JobInfoWrapper( ), errMex );
    
    break;
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_2_JobStatusInvalidFault:
    if( jr->union_2.JobStatusInvalidFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_2.JobStatusInvalidFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobInfoWrapper::JOBSTATUSINVALID, JobInfoWrapper( ), errMex );
    
    break; 
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_2_DelegationIdMismatchFault:
    if( jr->union_2.DelegationIdMismatchFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_2.DelegationIdMismatchFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobInfoWrapper::DELEGATIONIDMISMATCH, JobInfoWrapper( ), errMex );
    
    break; 
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_2_DateMismatchFault:
    if( jr->union_2.DateMismatchFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_2.DateMismatchFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobInfoWrapper::DATEMISMATCH, JobInfoWrapper( ), errMex );
    
    break; 
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_2_GenericFault:
    if( jr->union_2.GenericFault)
      errMex = ExceptionFactory::makeStringFromFault( jr->union_2.GenericFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobInfoWrapper::GENERIC, JobInfoWrapper( ), errMex );
    
    break;
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_2_LeaseIdMismatchFault:
    if( jr->union_2.LeaseIdMismatchFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_2.LeaseIdMismatchFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobInfoWrapper::LEASEIDMISMATCH, JobInfoWrapper( ), errMex );
    
    break;
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_2_jobInfo:
    if( jr->union_2.jobInfo ) {
      JobInfoWrapper jinfo( jr->union_2.jobInfo );
      (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobInfoWrapper::OK, jinfo, "" );
    }
    
    break;
    
  } // switch
} //  operator()

//______________________________________________________________________________
CreamProxy_Info::CreamProxy_Info( const JobFilterWrapper* filter, InfoArrayResult* target, const int timeout ) throw()
  : AbsCreamProxy( timeout ), m_jobFilter( filter ), m_target( target )
{
}

//_____________________________________________________________________________
void CreamProxy_Info::execute( const string& serviceAddress ) 
    throw(BaseException&,
          InvalidArgumentException&,
          GridProxyDelegationException&,
          JobSubmissionDisabledException&,
          GenericException&,
	  //          AuthenticationException&,
          AuthorizationException&,
          DelegationException&,
          InternalException&,
          ConnectionTimeoutException&,
          auth_ex&,
	  soap_ex&,
	  soap_runtime_ex&)
{
    static const char* method_name = "CreamProxy_Info::execute() - ";
    initSoap init(this);
  
#ifdef DOPROFILE
    apiUtil::scoped_timer T("CreamProxy_Info::execute-soap_call___CREAM__JobInfo");
#endif  
  
    _CREAMTYPES__JobInfoResponse resp;
    CREAM_SAFE_LOG(m_log_dev->debugStream() 
		   << "CreamProxy_Info::execute() - "
		   << "Sending JobInfo request with certificate ["
		   << m_certfile << "]"
		   );

    if(SOAP_OK != soap_call___CREAM__JobInfo(m_soap,
                                             serviceAddress.c_str(),
                                             NULL,    // soapAction
                                             (CREAMTYPES__JobFilter*)m_jobFilter, 
                                             &resp) ) {
        ExceptionFactory::raiseException(m_soap);
        
    } else {        
        m_target->clear();
        processorInfoResult proc(m_target);    
        for ( std::vector< CREAMTYPES__JobInfoResult* >::const_iterator it = resp.result.begin(); it != resp.result.end(); ++it ) {
            try {               
                proc(*it); //can raise any type of exception
            } catch( const std::exception& ex ) {
                CREAM_SAFE_LOG(m_log_dev->errorStream() << method_name
                               << "Got std::exception \""
                               << ex.what()
                               << "\". Rethrowing..."
			        );
                throw ex;
            } catch( ... ) {
                CREAM_SAFE_LOG(m_log_dev->errorStream() << method_name
                               << "Got unknown exception. Rethrowing..."
			        );
                throw; // rethrow
            }
        }
    }
}
