/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CreamProxy_Lease: the implementation of Lease class
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CreamProxy_Lease.cpp,v 1.6.4.1 2008/08/25 14:06:38 adorigo Exp $
*/

#include "CREAM_CLIENTH.h"
#include "CREAM_CLIENTStub.h"

#include "glite/ce/cream-client-api-c/CreamProxy_Impl.h"
#include "glite/ce/cream-client-api-c/creamApiLogger.h"

using namespace std;
using namespace glite::ce::cream_client_api::soap_proxy;

//______________________________________________________________________________
CreamProxy_Lease::CreamProxy_Lease( const pair<string, time_t>& Lease, pair<string, time_t>* target, const int timeout ) throw()
  : AbsCreamProxy( timeout ), m_leaseID( Lease.first ), m_leaseTime( Lease.second ), m_resp_lease( target )
{  
}

//______________________________________________________________________________
void CreamProxy_Lease::execute(const string& serviceURL)
  throw(BaseException&,
	InvalidArgumentException&,
	GridProxyDelegationException&,
	JobSubmissionDisabledException&,
	GenericException&,
	//	AuthenticationException&,
	AuthorizationException&,
	DelegationException&,
	InternalException&,
	ConnectionTimeoutException&,
	auth_ex&,
	soap_ex&,
	soap_runtime_ex&)
{

  initSoap init(this);

  CREAMTYPES__Lease leaseReq, leaseResp;
  leaseReq.leaseId = m_leaseID;
  leaseReq.leaseTime = m_leaseTime;

#ifdef DOPROFILE
  apiUtil::scoped_timer T("CreamProxy_Lease::execute()-soap_call___CREAM__setLease");
#endif
  
  if(SOAP_OK != soap_call___CREAM__setLease(m_soap,
					    serviceURL.c_str(),
					    NULL,// soapAction
					    &leaseReq,
					    &leaseResp))
    {
      
      ExceptionFactory::raiseException(m_soap);
      
    } else {
    
    m_resp_lease->first  = leaseResp.leaseId;
    m_resp_lease->second = leaseResp.leaseTime;
    
  }
  
}
