/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CreamProxy_List: the implementation of JobList class for the CREAM PROXY
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CreamProxy_List.cpp,v 1.6.4.2 2008/08/25 14:06:38 adorigo Exp $
*/

#include "CREAM_CLIENTH.h"
#include "CREAM_CLIENTStub.h"

#include "glite/ce/cream-client-api-c/CreamProxy_Impl.h"
#include "glite/ce/cream-client-api-c/creamApiLogger.h"

using namespace glite::ce::cream_client_api::soap_proxy;
using namespace std;

void processorListResult::operator()( const CREAMTYPES__JobId* aJob ) {
  if(aJob)
    m_target->push_back( JobIdWrapper(aJob) );
  
}

//______________________________________________________________________________
CreamProxy_List::CreamProxy_List(  vector<JobIdWrapper>* result,
				   const int timeout ) throw()
  : AbsCreamProxy( timeout), m_result( result )
{
}

//______________________________________________________________________________
void CreamProxy_List::execute( const std::string& serviceURL ) 
  throw(BaseException&,
	InvalidArgumentException&,
	GridProxyDelegationException&,
	JobSubmissionDisabledException&,
	GenericException&,
	//	AuthenticationException&,
	AuthorizationException&,
	DelegationException&,
	InternalException&,
	ConnectionTimeoutException&,
	auth_ex&,
	soap_ex&,
	soap_runtime_ex&)
{
  initSoap init(this);

  {
    _CREAMTYPES__JobListResponse resp;
#ifdef DOPROFILE
    apiUtil::scoped_timer T("CreamProxy_List::execute-soap_call___CREAM__JobList");
#endif  
    if(SOAP_OK != soap_call___CREAM__JobList(m_soap,
					      serviceURL.c_str(),
					      NULL,    // soapAction
					      &resp))
      ExceptionFactory::raiseException(m_soap);

    processorListResult processor( m_result );
    for_each(resp.result.begin(), resp.result.end(), processor);
  }
}
