/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CreamProxy_Status: the implementation of JobStatus class for the CREAM PROXY
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CreamProxy_Status.cpp,v 1.20.2.2 2008/08/25 14:06:38 adorigo Exp $
*/

#include "CREAM_CLIENTH.h"
#include "CREAM_CLIENTStub.h"

#include "glite/ce/cream-client-api-c/CreamProxy_Impl.h"
#include "glite/ce/cream-client-api-c/creamApiLogger.h"
#include "glite/ce/cream-client-api-c/JobStatusWrapper.h"
#include "glite/ce/cream-client-api-c/JobFilterWrapper.h"

#include <boost/tuple/tuple.hpp>

using namespace glite::ce::cream_client_api::soap_proxy;
using namespace std;

//__________________________________________________________________________________
void processorStatusResult::operator()( const CREAMTYPES__JobStatusResult* jr ) 
{
  
  if(!jr) 
    {
      throw InternalException("processorStatusResult::operator() - Pointer CREAMTYPES__JobStatusResult is NULL! Probable de-serialization error");
    }
  
  
  if(!jr->jobDescriptionId)
    {
      throw InternalException("processorStatusResult::operator() - Pointer CREAMTYPES__JobStatusResult->jobDescriptionId is NULL! Probable de-serialization error");
    }
  
  string errMex;
  
  switch( jr->__union_1) {
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_1_JobUnknownFault:
    if( jr->union_1.JobUnknownFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_1.JobUnknownFault );
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobStatusWrapper::JOBUNKNOWN, JobStatusWrapper( ), errMex );
    
    break;
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_1_JobStatusInvalidFault:
    if( jr->union_1.JobStatusInvalidFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_1.JobStatusInvalidFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobStatusWrapper::JOBSTATUSINVALID, JobStatusWrapper( ), errMex );
    
    break; 
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_1_DelegationIdMismatchFault:
    if( jr->union_1.DelegationIdMismatchFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_1.DelegationIdMismatchFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobStatusWrapper::DELEGATIONIDMISMATCH, JobStatusWrapper( ), errMex );
    
    break; 
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_1_DateMismatchFault:
    if( jr->union_1.DateMismatchFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_1.DateMismatchFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobStatusWrapper::DATEMISMATCH, JobStatusWrapper( ), errMex );
    
    break; 
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_1_GenericFault:
    if( jr->union_1.GenericFault)
      errMex = ExceptionFactory::makeStringFromFault( jr->union_1.GenericFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobStatusWrapper::GENERIC, JobStatusWrapper( ), errMex );
    
    break;
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_1_LeaseIdMismatchFault:
    if( jr->union_1.LeaseIdMismatchFault )
      errMex = ExceptionFactory::makeStringFromFault( jr->union_1.LeaseIdMismatchFault);
    else
      errMex = "Error Message Not Available";
    
    (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobStatusWrapper::LEASEIDMISMATCH, JobStatusWrapper( ), errMex );
    
    break;
    
    //------------------------------------------------------------
  case SOAP_UNION_CREAMTYPES__union_1_jobStatus:
    if( jr->union_1.jobStatus )
      (*m_target)[ *jr->jobDescriptionId ] = boost::make_tuple( JobStatusWrapper::OK, JobStatusWrapper( jr->union_1.jobStatus ), "" );
    
    break;
    
  } // switch
  
} // void operator()
  
//______________________________________________________________________________
CreamProxy_Status::CreamProxy_Status(  const JobFilterWrapper* filter, 
				       StatusArrayResult* target,
				       const int timeout ) throw()
  : AbsCreamProxy( timeout ), m_jobFilter( filter ), m_target( target )
{
}

//______________________________________________________________________________
void CreamProxy_Status::execute( const string& serviceAddress ) throw(BaseException&,
								      InvalidArgumentException&,
								      GridProxyDelegationException&,
								      JobSubmissionDisabledException&,
								      GenericException&,
								      // AuthenticationException&,
								      AuthorizationException&,
								      DelegationException&,
								      InternalException&,
								      ConnectionTimeoutException&,
								      auth_ex&,
								      soap_ex&,
								      soap_runtime_ex&)
{
  initSoap init(this);
  
#ifdef DOPROFILE
  apiUtil::scoped_timer T("CreamProxy_Status::execute-soap_call___CREAM__JobStatus");
#endif  
  
  _CREAMTYPES__JobStatusResponse resp;
  
  //cout << "CreamProxy_Status::execute() - Calling soap_call___CREAM__JobStatus to ["<<serviceAddress<<"]"<<endl;
  
  
  if(SOAP_OK != soap_call___CREAM__JobStatus(m_soap,
					     serviceAddress.c_str(),
					     NULL,    // soapAction
					     (CREAMTYPES__JobFilter*)m_jobFilter, 
					     &resp))
    {
      ExceptionFactory::raiseException(m_soap);
      
    } else {
    
    m_target->clear();
    processorStatusResult _processorResult(m_target);
    
    for_each( resp.result.begin(), resp.result.end(), _processorResult);
    
  }
}
