 /*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        JobIdWrapper: a wrapper for the new class JobDescription
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: JobIdWrapper.cpp,v 1.2.4.1 2008/08/26 07:54:11 adorigo Exp $
*/

#include "glite/ce/cream-client-api-c/JobIdWrapper.h"

using namespace glite::ce::cream_client_api::soap_proxy;
using namespace std;

//______________________________________________________________________________
void JobIdWrapper::clone( const CREAMTYPES__JobId* other ) throw()
{
    if ( other && other != this ) {
        destroy();
        id = other->id;
        creamURL = other->creamURL;
        for( vector<CREAMTYPES__Property*>::const_iterator it = other->property.begin(); it != other->property.end(); ++it ) {
            CREAMTYPES__Property* aProperty = new CREAMTYPES__Property();
            aProperty->value = (*it)->value;
            aProperty->name  = (*it)->name;
            property.push_back( aProperty );
        }
    }
}

void JobIdWrapper::destroy( void ) throw()
{
    vector<CREAMTYPES__Property*>::iterator it;
    for( it = property.begin(); it != property.end(); ++it ) {
        delete( *it );
    }
    property.clear();
}

//______________________________________________________________________________
JobIdWrapper& JobIdWrapper::operator=( const JobIdWrapper& aJobId ) throw() 
{   
    if ( &aJobId != this ) {
        clone( &aJobId );
    }
    return *this; 
}

//______________________________________________________________________________
JobIdWrapper::JobIdWrapper( const JobIdWrapper& aJob ) throw() 
  : CREAMTYPES__JobId()
{  
    clone( &aJob );
}

JobIdWrapper::JobIdWrapper( const CREAMTYPES__JobId* other ) 
{
    clone( other );
}

JobIdWrapper::JobIdWrapper( const string& CreamJobId, const string& CreamURL,
                            const vector<JobPropertyWrapper>& prop) throw() 
{
    id = CreamJobId;
    creamURL = CreamURL;            
    
    for( vector<JobPropertyWrapper>::const_iterator it = prop.begin(); it != prop.end(); ++it ) {
        CREAMTYPES__Property* aProperty = new CREAMTYPES__Property();
        aProperty->value = it->getPropertyName();
        aProperty->name  = it->getPropertyValue();
        property.push_back( aProperty );
    }
}

//______________________________________________________________________________
JobIdWrapper::~JobIdWrapper() throw() 
{
    destroy();
}

//______________________________________________________________________________
void JobIdWrapper::getProperties( map< string, string >& target ) 
{
    target.clear();
    vector<CREAMTYPES__Property*>::const_iterator it;
    for( it = property.begin(); it != property.end(); ++it ) {
        target[(*it)->name] = (*it)->value; 
    }
}
